<?php
/**
 * This Class gives tool to work with curl extension to communicate between
 * servers.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class Curl {

    private $headers;
    private $user_agent;
    private $compression;
    private $cookie_file;
    private $proxy;
    private $process;

    /**
     * This Class gives tool to work with curl extension to communicate between servers.
     * @param boolean $cookies enables cookies support
     * @param String $cookie local address for storing cookie files
     * @param String $compression used connection compression
     * @param String $proxy enables proxy and sets it up
     */
    public function __construct ( $cookies = true, $cookie = "cookies.db", $compression = "gzip", $proxy = "" )
    {
        $this -> headers [] = "Accept: image/gif, image/x-bitmap, image/jpeg, image/pjpeg";
        $this -> headers [] = "Connection: Keep-Alive";
        $this -> headers [] = "Content-type: application/x-www-form-urlencoded;charset=UTF-8";
        $this -> user_agent = $_SERVER [ "SERVER_SOFTWARE" ].' *'. $_SERVER [ "HTTP_HOST" ] . substr ( $_SERVER [ "SCRIPT_NAME" ], 0, strrpos ( $_SERVER [ "SCRIPT_NAME" ], "/" ) ) .'* (Albireo I version '. Tools::baseSettings () -> getSetting ( "version" ) .')';
        $this -> compression = $compression;
        $this -> proxy = $proxy;
        $this -> cookies = $cookies;
        if ( $this -> cookies == true ) $this -> cookie ( $cookie );
    }

    /**
     * destroyes the class instance and closes the connection
     */
    public function __destruct ()
    {
        curl_close( $this -> process );
    }

    /**
     * sets connection options
     * @param String $url target remote server url
     */
    private function CurlIni ( $url )
    {
        $this -> process = curl_init ();
        curl_setopt ( $this -> process, CURLOPT_URL, $url );
        curl_setopt ( $this -> process, CURLOPT_SSL_VERIFYPEER, 0 );
        curl_setopt ( $this -> process, CURLOPT_SSL_VERIFYHOST, 0 );
        //curl_setopt ( $this -> process, CURLOPT_SSLVERSION, 3 );
        curl_setopt ( $this -> process, CURLOPT_HTTPHEADER, $this -> headers );
        curl_setopt ( $this -> process, CURLOPT_HEADER, 0 );
        curl_setopt ( $this -> process, CURLOPT_USERAGENT, $this -> user_agent );
        if ( $this -> cookies == true ) curl_setopt ( $this -> process, CURLOPT_COOKIEFILE, $this -> cookie_file );
        if ( $this -> cookies == true ) curl_setopt ( $this -> process, CURLOPT_COOKIEJAR, $this -> cookie_file );
        curl_setopt ( $this -> process, CURLOPT_ENCODING , $this -> compression );
        curl_setopt ( $this -> process, CURLOPT_TIMEOUT, 30 );
        curl_setopt ( $this -> process, CURLOPT_CONNECTTIMEOUT, 20 );
        if ( $this -> proxy ) curl_setopt ( $this -> process, CURLOPT_PROXY, $this -> proxy );
        curl_setopt ( $this -> process, CURLOPT_RETURNTRANSFER, 1 );
        @curl_setopt ( $this -> process, CURLOPT_FOLLOWLOCATION, 1 );
    }

    /**
     * sets HTTP Authentication connection
     * @param String $login login:pasword
     */
    private function httpAuthentication ( $login )
    {
        curl_setopt ( $ch, CURLOPT_USERPWD, $login );
        curl_setopt ( $ch, CURLOPT_HTTPAUTH, CURLAUTH_ANY );
    }

    /**
     * saves cookie
     * @param String $cookie_file local address for storing cookie file
     * @return boolean success
     */
    private function cookie ( $cookie_file )
    {
        if ( file_exists ( $cookie_file ) ) {
            $this -> cookie_file = $cookie_file;
	    return true;
        } else {
            if ( !fopen ( $cookie_file, 'w' ) ) return false;
            $this -> cookie_file = $cookie_file;
            return true;
        }
    }

    /**
     * connects to remote server using GET method
     * @param String $url remote server url
     * @param String $httpauth HTTP Autgentication string
     * @return mixed returns the result on success, false on failure
     */
    public function get ( $url, $httpauth = "" ) {
        $this -> CurlIni ( $url );
        if ( $httpauth != "" ) $this -> httpAuthentication ( $httpauth );
        $return = curl_exec ( $this -> process );
        return $return;
    }

    /**
     * connects to remote server using POST method
     * @param String $url remote server url
     * @param String $data HTTP encoded POST data
     * @return returns the result on success, false on failure
     */
    public function post ( $url, $data ) {
        $this -> CurlIni ( $url );
        curl_setopt ( $this -> process, CURLOPT_POSTFIELDS, $data );
        curl_setopt ( $this -> process, CURLOPT_POST, 1 );
        $return = curl_exec ( $this -> process );
        return $return;
    }

    /**
     * Get information regarding a specific transfer
     * @return array returns an associative array with the following elements:
     * "url"
     * "content_type"
     * "http_code"
     * "header_size"
     * "request_size"
     * "filetime"
     * "ssl_verify_result"
     * "redirect_count"
     * "total_time"
     * "namelookup_time"
     * "connect_time"
     * "pretransfer_time"
     * "size_upload"
     * "size_download"
     * "speed_download"
     * "speed_upload"
     * "download_content_length"
     * "upload_content_length"
     * "starttransfer_time"
     * "redirect_time"
     */
    public function getinfo () {
        return curl_getinfo ( $this -> process );
    }

}
?>