<?php
/**
 * This class was developed to manage Albireo error handling. Works with design
 * pattern Singleton. You can access an instance by calling method ticket.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class Error {

    private static $instance = null;
    private $errors = array ();
    private $errors_secrets = array ();
    /**
     * contains highest error status of the session
     * 0    no error	    none
     * 1    warning	    orange
     * 2    error	    blue
     * 3    database error  green
     * 4    critical error  red
     * 5    security error  gold
     * @var number
     */
    public $status = 0;

    /**
     * Singleton method for calling Error instance
     * @return object
     */
    public static function ticket ()
    {
    	if ( self::$instance == null ) {
    	    self::$instance = new Error ();
    	}
    	return self::$instance;
    } // end ticket

    private function __construct ()
    {
    } // end __construct

    private function __clone ()
    {
    } // end __clone

    /**
     * saves Albireo errors
     * @param String $code error code
     * @param String $where where the error has occurred
     * @param String $secret error message only for administrators
     */
    public function notice ( $code, $where, $secret = "" )
    {
	if ( Tools::baseSettings () -> getSetting ( "primary_settings" ) == "true" ) {
	    $message = "";
	    if ( !file_exists ( "sqlite/errors.db" ) ) {
		$message = _( "Chyba při pokusu o vytvoření chybové hlášky: Databáze s chybami neexistuje." );
		$level = 4;
		$type = 7;
	    } else {
	    	if ( !$this -> dbh = @sqlite_open ( "sqlite/errors.db", 0666, $sqlite_error )  ) {
	    	    $message = _( "Chyba při pokusu o vytvoření chybové hlášky: Nelze se připojit k databázi s chybami - " ) . $sqlite_error;
	    	    $level = 4;
		    $type = 7;
		} else {
		    $result = @sqlite_query ( "SELECT level, type FROM errors WHERE code='". $code ."'", $this -> dbh );
		    $error_array = sqlite_fetch_array ( $result );
		    if ( $error_array [ "level" ] != null && $error_array [ "type" ] != null ) {
			$level = $error_array [ "level" ];
			$type = $error_array [ "type" ];
			if ( !$result = @sqlite_single_query ( "SELECT info FROM cztype WHERE type=". $type, $this -> dbh ) ) {
			    $message .= _( 'Chyba: Nelze vybrat typ chyby' ) ." ";
			    $secret .= "\n". _( "Typ chyby:" ) ." ". $type;
		   	}
		    	$message .= $result;
	
		    	if ( $result = @sqlite_single_query ( "SELECT info FROM czcode WHERE code='". $code ."'", $this -> dbh ) ) {
			    $message .= " : ". $result;
		    	} else {
		    	    $message .= " : ". _( "Chyba: Nelze vybrat info chyby" );
		    	    $secret .= "\n". _( "Kód chyby:" ) ." ". $code;
		    	}
		    } else {
		    	$message = _( "Chyba: Nelze vybrat chybu z databáze" );
		    	$secret .= "\n". _( "Kód chyby:" ) ." ". $code;
		    	$level = 4;
		    	$type = 7;
		    }
		}
	    }

    	    /* *** error level table **************
    	    0	no error		none
    	    1	warning			orange
    	    2	error			blue
    	    3	database error		green
    	    4	critical error		red
    	    5	security error		gold
    	    ************************************ */
    	    switch ( $level ) {
    	    	case 1:
    	    	    $color = ' class="orange"';
    	    	    break;
    	    	case 2:
    	    	    $color = ' class="blue"';
    	    	    break;
    	    	case 3:
    	    	    $color = ' class="green"';
    	    	    break;
    	    	case 4:
    	    	    $color = ' class="red"';
    	    	    break;
    	    	case 5:
    	    	    $color = ' class="gold"';
    	    	    break;
    	    }

    	    $this -> errors [ $message ] = $color;
    	    $this -> errors_secrets [ $where ] = $secret .' '. $where;

	    if ( Database::connection () -> is_connected () ) {
		$this -> log ($code, $where, $secret, $type, $level);
	    }

    	    if ( $this -> status < $level ) {
    	    	$this -> status = $level;
    	    	if ( $level > 3 ) {
		    if ( $level == 5 ) {
		    	$_SESSION = array();
        		session_destroy();
		    }

		    if ( substr ( $_SERVER [ "SCRIPT_NAME" ], -7 ) == "xml.php" ) {
		    	XMLPublisher::publish ();
		    } else {
		    	XHTMLPublisher::publish ();
		    }
		    exit;
		}
    	    }
	 }
    } // end notice
	
    private function log ( $code, $where, $secret, $type, $level)
    {
	$ip = $_SERVER [ "REMOTE_ADDR" ];

	if ($type!=2) {
	    Database::connection () -> query ("INSERT INTO logerrors (code,time,userid,ip,agent,position,secret,type,level) VALUES ('".$code."',NOW(),".User::session () -> getId ().",'".$ip."','".$_SERVER['HTTP_USER_AGENT']."','".$where."','".$secret."',".$type.",".$level.") ");
	}	
    } // end log

    /**
     * returns a string containing list of errors
     * @return String
     */
    public function getErrors ()
    {
    	$error = '<ul>';
    	if ( User::session () -> getPrimaryGroup () != 1  ) {
    	    foreach ( $this -> errors as $chyba => $color ) {
    	    	$error .= '<li'.$color.'><strong>'. $chyba .'</strong></li>';
    	    }
	    if ( isset ( $this -> chyba ) ) {
		$error .= '<li><strong>'. $this->chyba .'</strong></li>';
	    }
    	} else {
    	    foreach ( $this -> errors as $chyba => $color ) {
    	    	$error .= '<li'.$color.'><strong>'.$chyba.'</strong> <br />'.current ( $this -> errors_secrets ).'</li>';
    	    	next ( $this -> errors_secrets );
    	    }
 	    if ( isset ( $this -> chyba ) ) {
		$error .= '<li><strong>'. $this->chyba .'</strong></li>';
	    }
			
    	}
    	return $error.'</ul>';	
    } // end getErrors
    
}

?>