<?php
/**
 * Class ErrorPage is called by Page class after fatal or security error
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class ErrorPage implements PageInterface {

    private $page;

    /**
     * creates webpage definition object for error handling
     */
    public function __construct ()
    {

    	if ( Error::ticket () -> status == 4 ) {
    	    $this -> page [ "title" ] = _( "Kritická chyba" );
    	    $this -> page [ "editorial" ] = _( "Kritická chyba zachycená na webových stránkách." );
    	    $this -> page [ "text" ] = "<p>". _( "Při běhu stránek se objevila kritická chyba, která znemožnila běžný běh webu." ) ."</p>
    	    				<p>". _( "Omlouváme se za případné potíže a ujišťujeme Vás, že uděláme maximum, abychom problém odstranily." ) ."</p>
    	    				<p>". _( "Automatika systému už nyní udělala maximum pro informování administrátorů." ) ."</p>";
    	} else {
    	    $this -> page [ "title" ] = _( "Bezpečnostní chyba" );
    	    $this -> page [ "editorial" ] = _( "Bezpečnostní chyba zachycená na webových stránkách." );
    	    $this -> page [ "text" ] = "<p>". _( "Vaše činnost na stránkách vyvolala bezpečnostní chybu. Neopakujte prosím znovu akci,
    	    				kterou jste právě provedli." ) ."</p>
    	    				<p>". _( "Automatika systému už nyní udělala maximum pro informování administrátorů." ) ."</p>
    	    				<p>". _( "Pokud byla bezpečnostní chyba vyvolána záměrně, upozorňujeme Vás, že jsme uložili maximum
    	    				informací, které by mohly být použity k vaší identifikaci." ) ."</p>";
    	}

    } // end __construct

    /**
     * returns page type (static)
     * @return String
     */
    public function getType ()
    {
    	return $this -> page [ "static" ];
    } // end getType

    /**
     * returns id of page owner who is administrator
     * @return number id
     */
    public function getAuthor ()
    {
    	return 1;
    }

    /**
     * returns page access code (author,group,others)
     * @return char(3) rrr
     */
    public function getAccess ()
    {
    	return "rrr";
    }

    /**
     * returns user location status (web)
     * @return String
     */
    public function getStatus ()
    {
    	return "web";
    }

    /**
     * returns web page text
     * @return String
     */
    public function getText ()
    {
    	return $this -> page [ "text" ];
    }

    /**
     * returns web page title
     * @return String
     */
    public function getTitle ()
    {
    	return $this -> page [ "title" ];
    }

    /**
     * returns web page editorial
     * @return String
     */
    public function getEditorial ()
    {
    	return $this -> page [ "editorial" ];
    }

    /**
     * returns directory id
     * @return number id
     */
    public function getDirectory ()
    {
    	return 1;
    } // end getDirectory

    /**
     * returns published datetime
     * @return void
     */
    public function getPublished ()
    {
    	return 0;
    } // end getPublished

}
?>