<?php
/**
 * Provides an OpenID tool. Based on Simple OpenID PHP Class.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class OpenId {
    private $openid_url_identity;
    private $URLs = array ();
    private $error = array ();
    private $connections_loop = 1;
    private $fields = array (
	    'required' => array (),
	    'optional' => array ());

    /**
     * class contruct
     */
    public function __construct () {
    }

    /**
     * sets OpenID identity url
     * @param String $identity_url identity url
     */
    public function SetIdentity ( $identity_url ) {
	if ( ( stripos ( $identity_url, 'http://' ) === false ) && ( stripos ( $identity_url, 'https://' ) === false ) ){
	    $identity_url = 'http://'.$identity_url;
	}

	$parsed_url = parse_url ( trim ( $identity_url ) );
	if ( !isset ( $parsed_url [ 'path' ] ) ) {
	    $parsed_url [ 'path' ] = '/';
	} elseif ( substr ( $parsed_url [ 'path' ], -1, 1 ) == '/' ) {
	    $parsed_url [ 'path' ] = substr ( $parsed_url [ 'path' ], 0, strlen ( $parsed_url [ 'path' ] ) -1 );
	}
	if ( isset ( $parsed_url [ 'query' ] ) ) { // If there is a query string, then use identity as is
	    $identity = $identity_url;
	} else {
	    $identity = $parsed_url [ 'scheme' ] . '://' . $parsed_url [ 'host' ] . $parsed_url [ 'path' ];
	}

	$this -> openid_url_identity = str_replace ( "http://", "https://", $identity );
    }

    /**
     * sets OpenID trust root (URL of our webpage)
     * @param String $url sets trust root
     */
    public function SetTrustRoot ( $url ){
	$this -> URLs [ 'trust_root' ] = $url;
    }

    /**
     * contact OpenID server using cURL (3 attempts)
     * @return mixed false or OpenID server
     */
    public function GetOpenIDServer() {
	$response = $this -> CURL_Request ( $this -> openid_url_identity );
	while ( $response == null && $this -> connections_loop < 3 ) {
	    $this -> connections_loop = $this -> connections_loop + 1;
	    $response = $this -> CURL_Request ( $this -> openid_url_identity );
	}
	if ( $response == null ) {
	    $this -> ErrorStore ( 'REMOTE_SERVER', _('Komunikace s Vaším OpenID serverem selhala.') );
	    return false;
	}
	list ( $servers, $delegates ) = $this -> HTML2OpenIDServer ( $response );
	if ( count ( $servers ) == 0 ) {
	    $this -> ErrorStore ( 'OPENID_NOSERVERSFOUND' );
	    return false;
	}
	if ( isset ( $delegates [ 0 ] ) && ( $delegates [ 0 ] != "" ) ){
	    $this -> SetIdentity ( $delegates [ 0 ] );
	}
	$this -> SetOpenIDServer ( $servers [ 0 ] );
	return $servers [ 0 ];
    }

    private function CURL_Request ( $url, $method = "GET", $params = "" ) { // Remember, SSL MUST BE SUPPORTED
	if ( is_array ( $params ) ) $params = $this -> array2url ( $params );
	$curl = curl_init ( $url . ( $method == "GET" && $params != "" ? "?" . $params : "" ) );
	@curl_setopt ( $curl, CURLOPT_FOLLOWLOCATION, true );
	curl_setopt ( $curl, CURLOPT_RETURNTRANSFER, true );
	curl_setopt ( $curl, CURLOPT_HEADER, false );
	curl_setopt ( $curl, CURLOPT_SSL_VERIFYPEER, false );
	curl_setopt ( $curl, CURLOPT_SSL_VERIFYHOST, false );
	curl_setopt ( $curl, CURLOPT_TIMEOUT, 30 );
	curl_setopt ( $curl, CURLOPT_CONNECTTIMEOUT, 20 );
	//curl_setopt ( $curl, CURLOPT_SSLVERSION, 3 );
	if ( $method == "POST" ) {
	    curl_setopt ( $curl, CURLOPT_POST, ( $method == "POST" ) );
	    curl_setopt ( $curl, CURLOPT_POSTFIELDS, $params );
	} else {
	    curl_setopt ( $curl, CURLOPT_HTTPGET, ( $method == "GET" ) );
	}
	$response = curl_exec ( $curl );
	if ( $response == null ) {
	    $this -> ErrorStore ( 'REMOTE_SERVER', 'Komunikace s Vaším OpenID serverem selhala.' );
	}
	if ( curl_errno ( $curl ) != 0 ){
	    $this -> ErrorStore ( 'OPENID_CURL', curl_error ( $curl ) );
	}
	return $response;
    }

    private function HTML2OpenIDServer ( $content ) {
	$get = array();

	// Get details of their OpenID server and (optional) delegate
	preg_match_all ( '/<link[^>]*rel=[\'"]openid.server[\'"][^>]*href=[\'"]([^\'"]+)[\'"][^>]*\/?>/i', $content, $matches1 );
	preg_match_all ( '/<link[^>]*href=\'"([^\'"]+)[\'"][^>]*rel=[\'"]openid.server[\'"][^>]*\/?>/i', $content, $matches2 );
	preg_match_all ( '/<link[^>]*rel=[\'"]openid2.provider openid.server[\'"][^>]*href=[\'"]([^\'"]+)[\'"][^>]*\/?>/i', $content, $matches3 );
	preg_match_all ( '/<link[^>]*rel=[\'"]openid2.provider[\'"][^>]*href=[\'"]([^\'"]+)[\'"][^>]*\/?>/i', $content, $matches4 );
	$servers = array_merge ( $matches1 [ 1 ], $matches2 [ 1 ], $matches3 [ 1 ], $matches4 [ 1 ] );

	preg_match_all ( '/<link[^>]*rel=[\'"]openid.delegate[\'"][^>]*href=[\'"]([^\'"]+)[\'"][^>]*\/?>/i', $content, $matches1 );
	preg_match_all ( '/<link[^>]*href=[\'"]([^\'"]+)[\'"][^>]*rel=[\'"]openid.delegate[\'"][^>]*\/?>/i', $content, $matches2 );
	$delegates = array_merge ( $matches1 [ 1 ], $matches2 [ 1 ] );

	$ret = array ( $servers, $delegates );
	return $ret;
    }

    private function SetOpenIDServer ( $a ) {
	$this -> URLs [ 'openid_server' ] = $a;
    }

    private function array2url ( $arr ) { // converts associated array to URL Query String
	if ( !is_array ( $arr ) ){
	    return false;
	}
	$query = '';
	foreach ( $arr as $key => $value ){
	    $query .= $key . "=" . $value . "&";
	}
	return $query;
    }

    /**
     * sets approved url
     * @param String $url approved url
     */
    public function SetApprovedURL ( $url ) {
	$this -> URLs [ 'approved' ] = $url;
    }

    /**
     * redirects to target page after login
     */
    public function Redirect () {
	$redirect_to = $this -> GetRedirectURL ();
	if ( headers_sent () ) { // Use JavaScript to redirect if content has been previously sent (not recommended, but safe)
	    echo '<script language="JavaScript" type="text/javascript">window.location=\'';
	    echo $redirect_to;
	    echo '\';</script>';
	} else { // Default Header Redirect
	    header ( 'Location: ' . $redirect_to );
	}
    }

    /**
     * sets cancel url
     * @param String $url url
     */
    public function SetCancelURL ( $url ){
	$this -> URLs [ 'cancel' ] = $url;
    }


    /**
     * sets required fields for url
     * @param ? $a ?
     */
    public function SetRequiredFields($a){
	if (is_array($a)){
	    $this->fields['required'] = $a;
	} else {
	    $this->fields['required'][] = $a;
	}
    }

    /**
     * sets optional fields for url
     * @param ? $a ?
     */
    public function SetOptionalFields($a){
	if (is_array($a)){
	    $this->fields['optional'] = $a;
	} else {
	    $this->fields['optional'][] = $a;
	}
    }

    /**
     * shows OpenID identity url
     * @return String url
     */
    public function GetIdentity () {
	return $this -> openid_url_identity;
    }

    /**
     * returns errors
     * @return array associative array with errors:
     * "code",
     * "description"
     */
    public function GetError (){
	$e = $this -> error;
	return array ( 'code' => $e [ 0 ], 'description' => $e [ 1 ] );
    }

    private function ErrorStore ( $code, $desc = null ) {
	$errs [ 'OPENID_NOSERVERSFOUND' ] = 'Nelze nalézt značku OpenID serveru na stránce identity.';
	if ( $desc == null ){
	    $desc = $errs [ $code ];
	}
	$this -> error = array ( $code, $desc );
    }

    /**
     * returns error status
     * @return boolean
     */
    public function IsError(){
	if (count($this->error) > 0){
	    return true;
	} else {
	    return false;
	}
    }

    private function splitResponse($response) {
	$r = array();
	$response = explode("\n", $response);
	foreach($response as $line) {
	    $line = trim($line);
	    if ($line != "") {
		list($key, $value) = explode(":", $line, 2);
		$r[trim($key)] = trim($value);
	    }
	}
	return $r;
    }

    /**
     * standartize OpenID identity url
     * @param String $openid_identity OpenID identity url
     * @return String standartized OpenID identity url
     */
    public function OpenID_Standarize($openid_identity = null){
	if ($openid_identity === null) $openid_identity = $this->openid_url_identity;

	$u = parse_url(strtolower(trim($openid_identity)));

	if (!isset($u['path']) || ($u['path'] == '/')) {
	    $u['path'] = '';
	}
	if(substr($u['path'],-1,1) == '/'){
	    $u['path'] = substr($u['path'], 0, strlen($u['path'])-1);
	}
	if (isset($u['query'])){ // If there is a query string, then use identity as is
	    return $u['host'] . $u['path'] . '?' . $u['query'];
	}else{
	    return $u['host'] . $u['path'];
	}
    }

    private function GetRedirectURL(){
	$params = array();
	$params['openid.return_to'] = urlencode($this->URLs['approved']);
	$params['openid.mode'] = 'checkid_setup';
	$params['openid.identity'] = urlencode($this->openid_url_identity);
	$params['openid.trust_root'] = urlencode($this->URLs['trust_root']);

	if (isset($this->fields['required'])
	  && (count($this->fields['required']) > 0)) {
		$params['openid.sreg.required'] = implode(',',$this->fields['required']);
	}
	if (isset($this->fields['optional'])
	  && (count($this->fields['optional']) > 0)) {
		$params['openid.sreg.optional'] = implode(',',$this->fields['optional']);
	}
	return $this->URLs['openid_server'] . "?". $this->array2url($params);
    }

    /**
     * validates OpenID login (authenticates)
     * @return boolean
     */
    public function ValidateWithServer() {
	$params = array(
		'openid.assoc_handle' => urlencode ( $_GET [ 'openid_assoc_handle' ] ),
		'openid.signed' => urlencode ( $_GET [ 'openid_signed' ] ),
		'openid.sig' => urlencode ( $_GET [ 'openid_sig' ] )
	);
	// Send only required parameters to confirm validity
	$arr_signed = explode ( ",", str_replace ( 'sreg.', 'sreg_', $_GET [ 'openid_signed' ] ) );
	for ( $i = 0; $i < count ( $arr_signed ); $i++ ){
	    $s = str_replace ( 'sreg_', 'sreg.', $arr_signed [ $i ] );
	    $c = $_GET [ 'openid_' . $arr_signed [ $i ] ];
	    // if ($c != ""){
		    $params [ 'openid.' . $s ] = urlencode ( $c );
	    // }
	}
	$params [ 'openid.mode' ] = "check_authentication";

	$openid_server = $this -> GetOpenIDServer ();
	if ( $openid_server == false ){
	    return false;
	}
	$response = $this -> CURL_Request ( $openid_server, 'POST', $params );
	$data = $this -> splitResponse ( $response );

	if ( $data [ 'is_valid' ] == "true" ) {
	    return true;
	} else {
	    return false;
	}
    }
}
?>