<?php
/**
 * This is interface for working with classes managing web pages
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
interface PageInterface {

    /**
     * returns page type
     * @return String (static|dynamic|script)
     */
    public function getType ();

    /**
     * returns id of page owner (in database table users)
     * @return number id
     */
    public function getAuthor ();

    /**
     * returns user location status
     * @return String
     */
    public function getStatus ();

    /**
     * returns page access code (author,group,others)
     * @return char(3)
     */
    public function getAccess ();

    /**
     * returns web page text
     * @return String
     */
    public function getText ();

    /**
     * returns web page title
     * @return String
     */
    public function getTitle ();

    /**
     * returns web page editorial
     * @return String
     */
    public function getEditorial ();

    /**
     * returns directory id (in database table directory)
     * @return number id
     */
    public function getDirectory ();

    /**
     * returns published datetime
     * @return datetime
     */
    public function getPublished ();

}
?>