<?php
/**
 * Parses all Albireo XHTML files, extends Publisher.  Works with design
 * pattern Singleton.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class Parser extends Publisher {

    // TODO: add phpDoc
    public static $err;

    private static $instance = null;

    /**
     * Singleton method for calling Parser instance
     * @return object
     */
    public static function session ()
    {
        self::$instance = new Parser ();
    	return self::$instance;
    } // end session

    private function __construct ()
    {
    } // end __construct

    private function __clone ()
    {
    } // end __clone

    /**
     * starts parsing
     * @param String $resource file address to parse
     * @param String $element element to replace
     * @return String parsed file
     */
    public function parserStart ( $resource, $element )
    {    	
	$this -> data_array = null;
	$this -> attr_array = null;
	$this -> i = 0;
	$this -> opened = 0;
	$analysis_handler = xml_parser_create ();
	xml_set_object ( $analysis_handler, $this );
	xml_parser_set_option( $analysis_handler, XML_OPTION_CASE_FOLDING, 0);
    
	if ( empty ( $this -> parse_only ) ) {
    	    xml_set_element_handler ( $analysis_handler, "parseAllToArrayBegin", "parseAllToArrayEnd" );
	    xml_set_character_data_handler ( $analysis_handler, "parseAllToArrayData" );
	} else {
    	    xml_set_element_handler ( $analysis_handler, "parseOnlyBegin", "parseOnlyEnd" );
	    xml_set_character_data_handler ( $analysis_handler, "parseOnlyData" );
	}
	$xml = $this -> loadData ( $resource );
	$xml = $this -> htmlentities2unicodeentities ($xml);
	
	if ( !xml_parse ( $analysis_handler, $xml ) ) {
	    self::$err = $this -> xmlError ( $analysis_handler );
    	}
	if ( empty ( self::$err ) ) {
            return $this -> seekAndReplace ( $element );
	} else {
	    return false;
	}
    } // end parserStart
	
    private function htmlentities2unicodeentities ($input)
    {
	$htmlEntities = array_values (get_html_translation_table (HTML_ENTITIES, ENT_QUOTES));
	$entitiesDecoded = array_keys   (get_html_translation_table (HTML_ENTITIES, ENT_QUOTES));
	$num = count ($entitiesDecoded);
	for ($u = 0; $u < $num; $u++) {
	    $utf8Entities[$u] = '&#'.ord($entitiesDecoded[$u]).';';
	}
	return str_replace ($htmlEntities, $utf8Entities, $input);
    }

    // TODO: add phpDoc
    protected function seekAndReplace ( $element )
    { 
	$out = "";
	$replace = "";
	$counter = 0;
	$i = 0;
	$dont_write = false;
	foreach ( $this -> data_array as $data ) {
	    if ( $data == ".".$element ) {
		$counter++;
	    }
	   if ( $counter > 0 ) {
		if ( $data == "/" . $element ) {
		    $out .= $this -> lookFor ( $replace, $my_array );
		    $my_array = null;
		    $replace = "";
		} elseif ( ( $data != "." . $element ) and ( !empty( $data ) ) ) {
		    $replace .= $data;
		    if ( is_array ( $this -> attr_array [ $i ] ) ) {
			foreach ( $this -> attr_array [ $i ] as $nam => $val ) {
			    $my_array [ $nam ] = $val;
			}
			$replace .= ">";
		    }
		}
	    }
	    if ( !$dont_write ) {
		if ( $counter <= 0 ) {
		    if ( ( ! empty( $data ) ) and ( ( substr( $data, 0, 1 ) == "." ) or ( substr( $data, 0, 1 ) == "/" ) ) ) {
			if ( ( substr( $data,0,1 ) == "." ) ) {
			    $data = substr( $data,1 );
			}
			$out .= "<". $data;
			if ( ! empty ( $this -> attr_array [ $i ] ) ) {
			    foreach ( $this -> attr_array [ $i ] as $nam => $val ) {
				$out .= " " . $nam . '="' .  str_replace ( '"', "'", $val ) .'"';
			    }
			}
			if ( ( substr ( $this -> data_array [ $i ], 0, 1 ) == "." ) and ( substr ( $this -> data_array [ $i + 1 ], 0, 1 ) == "/" )  ) {
			    $out .= " /> \n";
			    $dont_write = true;
			} else {
			    $out .= "> \n";
			}
		    } else {
			$out .= $data;
		    }
		}
	    } else {
		$dont_write = false;
	    }
	    if ( $data == ( "/". $element ) ) {
		$counter--;
	    }
	    $i++;
	}
	return $out;
    } // end seekAndReplace

    // TODO: add phpDoc
    protected function lookFor ( $ext, $attributes )
    {
	$pos = strpos ( $ext, '>' );
	if ( $pos > 0 ) {
    	    $name = substr ( $ext , 1 , $pos-1 );
    	}	
	$pos = strpos ( $name, ' ' );
	if ( $pos > 0 ) {
    	    $name = substr ( $name , 0 , $pos-1 );
    	}

    	$current_result = Database::connection () -> getResult ( "SELECT file FROM extensiontags WHERE element='$name';" );

    	if ( empty ( $current_result ) ) {
	    if ( is_array ( $attributes ) ) {
		foreach ( $attributes as $tit => $value ) {
		    if ( $tit == "notfound" ) {
		    	return str_replace ( "]", ">", str_replace ( "[", "<", $value ) );
		     }
		}
	    }
        } else {
	    $myclass = "ext_". $name;
	    $universal_object = new $myclass;
	    $val = $universal_object -> extension ( $attributes );
	}
	return $val;
    } // end lookFor
}
?>