﻿<?php
/**
 * This class is used to parse XML strings or files.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class Publisher {

    /**
     * all elemnent and data are stored here. Starting element is signed by dot ( . ), endind element by slash ( / ), data are not marked
     * @var array
     */
    public $data_array;

    /**
     * all attributes are stored here as array (it  has same number as the element-owner e.g. if element is $this->data_array[5] then its attributes are here: $this->attr_array[5]["whatever"]
     * @var array
     */
    public $attr_array;

    /**
     * if you want to parse only content of some element, sets here its name
     * @var String
     */
    public  $parse_only;

    private $opened;
    private $i;

    /**
     * is used to begin parsing file or string determined in $resource
     * @param resource $resource data resource
     */
    public function parserStart ( $resource )
    {
	$this -> data_array = null;
	$this -> attr_array = null;
	$this -> i = 0;
	$this -> opened = 0;
	$analysis_handler = xml_parser_create ();
	xml_set_object ( $analysis_handler, $this );
	xml_parser_set_option($analysis_handler,XML_OPTION_CASE_FOLDING,0);

	if ( empty ( $this -> parse_only ) ) {
	    xml_set_element_handler ( $analysis_handler, "parseAllToArrayBegin", "parseAllToArrayEnd" );
	    xml_set_character_data_handler ( $analysis_handler, "parseAllToArrayData" );
	} else {
	    xml_set_element_handler ( $analysis_handler, "parseOnlyBegin", "parseOnlyEnd" );
	    xml_set_character_data_handler ( $analysis_handler, "parseOnlyData" );
	}

	$xml = $this -> loadData ( $resource );
	if ( !xml_parse ( $analysis_handler, $xml ) ) {
	    $this->xmlError($analysis_handler);
	}
    } // end parserStart

    // TODO: add phpDoc
    protected function loadData ( $resource )
    {
	if ( file_exists ( $resource ) ) {
	    return file_get_contents ( $resource, 0);
	} else {
	    return $resource;
    	}
    } // end loadData

    // TODO: add phpDoc
    protected function parseAllToArrayBegin ( $analysis_handler, $tag_name, $attributes )
    {
	$this -> data_array [ $this -> i ] = "." . $tag_name;
	foreach ( $attributes as $name => $value ) {
	    $this -> attr_array [ $this -> i ] [ $name ] = $value;
    	}
    	$this -> i++;
    } // end parseAllToArrayBegin

    // TODO: add phpDoc
    protected function parseAllToArrayEnd ( $analysis_handler, $tag_name )
    {
     	$this -> data_array [ $this -> i ] = "/". $tag_name;
    	$this -> i++;
    } // end parseAllToArrayEnd

    // TODO: add phpDoc
    protected function parseAllToArrayData ( $analysis_handler, $data )
    {
	$this -> data_array [ $this -> i ] = ltrim ( $data );
	$this -> i++;
    } // end parseAllToArrayData

    // TODO: add phpDoc
    protected function parseOnlyBegin ( $analysis_handler, $tag_name, $attributes )
    {   
	if ( $tag_name == $this -> parse_only ) {
    	    $this -> opened++;
    	}
    	if ( $this -> opened > 0 ) {
	    $this -> data_array [ $this -> i ] = ".". $tag_name;
	    foreach ( $attributes as $name => $value ) {
		$this -> attr_array [ $this -> i ] [ $name ] = $value;
	    }
	    $this -> i++;
	}
    } // end parseOnlyBegin

    // TODO: add phpDoc
    protected function parseOnlyEnd ( $analysis_handler, $tag_name )
    {
	if ( $this -> opened > 0 ) {
	    $this -> data_array [ $this -> i ] = "/". $tag_name;
	    $this -> i++;
	}
	if ( $tag_name == $this -> parse_only ) {
	    $this -> opened--;
    	}
    } // end parseOnlyEnd

    // TODO: add phpDoc
    protected function parseOnlyData ( $analysis_handler, $data )
    {
    	if ( $this -> opened > 0 ) {
	    $this -> data_array [ $this -> i ] = trim ( $data );
	    $this -> i++;
	}
    } // end parseOnlyData

    // TODO: add phpDoc
    protected function xmlError ( $analysis_handler )
    {
        return "ERROR: "
              . xml_error_string(xml_get_error_code( $analysis_handler ))
              . "<br />"
              . "Line: "
              . xml_get_current_line_number( $analysis_handler )
              . "<br />"
              . "Column: "
              . xml_get_current_column_number( $analysis_handler )
              . "<br />";

    } // end xmlError
}
?>