<?php
/**
 * Class ScriptPage is called by class Page for managing pages combinating MySQL
 * database and dynamic php scripts.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class ScriptPage implements PageInterface {

    private $page;

    /**
     * Class ScriptPage is called by class Page for managing pages combinating MySQL database and dynamic php scripts.
     * @param String $getPage page file attribute
     */
    public function __construct ( $getPage = "null" )
    {

	$getPage = ( $getPage == "null" ) ? Tools::getPage () : $getPage;
	
	$id_file = Database::connection () -> getResult ( "SELECT id FROM header WHERE file='$getPage';" );

	$this -> page = Database::connection () -> fetch_assoc ( "SELECT * FROM header, content WHERE header.id=$id_file AND content.id_header=$id_file;" ); // private variable page will contain associative array corresponding to database tables header and content

	// access checking ///////////////////////////////////////////////////////////
	if ( $this -> page [ "directory" ] == 0 ) { // page is in no directory, only article access will be checked
	    if ( !SecurityTools::check_read ( $this -> page [ "access" ], $this -> page [ "author" ], $this -> page [ "groupid" ] ) ) {
	    	if ( $this -> page [ "directory" ] == 1 || Database::connection () -> getResult ( "SELECT parent FROM directory WHERE id=". $this -> page [ "directory" ] .";" ) == 1 ) {
	    	    header ( "Location:". Localization::session () -> getFile ( "cor-log-in" ) ."?loginpage=". Tools::getPage () );
	    	} else {
	    	    Error::ticket () -> notice ( 'cor00028', 'in '. __FILE__ .' on line '. __LINE__ , $getPage ." ". User::session () -> getNick () );
	    	}
	    }
	} else { // system will check article and directory access
	    if ( !SecurityTools::check_read ( $this -> page [ "access" ], $this -> page [ "author" ], $this -> page [ "groupid" ] ) ) {
	    	if ( $this -> page [ "directory" ] == 1 || Database::connection () -> getResult ( "SELECT parent FROM directory WHERE id=". $this -> page [ "directory" ] .";" ) == 1 ) {
	    	    header ( "Location:". Localization::session () -> getFile ( "cor-log-in" ) ."?loginpage=". Tools::getPage () );
	    	} else {
	    	    Error::ticket () -> notice ( 'cor00028', 'in '. __FILE__ .' on line '. __LINE__ , $getPage ." ". User::session () -> getNick () );
	    	}
	    }
	}
	//////////////////////////////////////////////////////////////////////////////

	$base_directory = $this -> page [ "directory" ];
	$this -> page [ "status" ] = "web";
	while ( $base_directory != 0 ) {
	    if ( $base_directory == 1 ) {
	    	$this -> page [ "status" ] = "admin";
	    }
	    $base_directory = Database::connection () -> getResult ( "SELECT parent FROM directory WHERE id=$base_directory;" );
	}

	$script_file = Database::connection () -> getResult ( "SELECT script FROM scriptpages WHERE id_header=$id_file;" );
	// require requested php script //////////////////////////////////////////////
	if ( file_exists ( "$script_file.php" ) ) { // check if the file really exists
	    require_once ( "$script_file.php" );  // contains $page containning changes for $this -> page
	    if ( is_array ( $page ) ) { // check script security
	    	foreach ( $page as $key => $value ) {
	    	    $this -> page [ $key ] = $value;
	    	}
	    } else {
	    	Error::ticket () -> notice ( 'cor00020', 'in '. __FILE__ .'-'. __CLASS__ .' on line '. __LINE__, "$script_file.php" );
	    }
	} else {
	    Error::ticket () -> notice ( 'cor00021', 'in '. __FILE__ .'-'. __CLASS__ .' on line '. __LINE__, "$script_file.php" );
	}
	//////////////////////////////////////////////////////////////////////////////

    } // end __construct

    /**
     * returns page type
     * @return String (static|dynamic|script)
     */
    public function getType ()
    {
    	return $this -> page [ "type" ];
    } // end getType

    /**
     * returns id of page owner (in database table users)
     * @return number id
     */
    public function getAuthor ()
    {
    	return $this -> page [ "author" ];
    }

    /**
     * returns page access code (author,group,others)
     * @return char(3)
     */
    public function getAccess ()
    {
    	return $this -> page [ "access" ];
    }

    /**
     * returns user location status
     * @return String
     */
    public function getStatus ()
    {
    	return $this -> page [ "status" ];
    }

    /**
     * returns web page text
     * @return String
     */
    public function getText ()
    {
    	if ( $this -> page [ "text" ] == "0" ) {
    	    return "";
    	} else {
    	    return $this -> page [ "text" ];
    	}
    }

    /**
     * returns web page title
     * @return String
     */
    public function getTitle ()
    {
    	return $this -> page [ "title" ];
    }

    /**
     * returns web page editorial
     * @return String
     */
    public function getEditorial ()
    {
    	return $this -> page [ "editorial" ];
    }

    /**
     * returns directory id (in database table directory)
     * @return number id
     */
    public function getDirectory ()
    {
    	return $this -> page [ "directory" ];
    } // end getDirectory

    /**
     * returns published datetime
     * @return datetime
     */
    public function getPublished ()
    {
    	return $this -> page [ "published" ];
    } // end getPublished
    
}

?>