<?php
/**
 * Contains basic static methods to handle Albireo access checks
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class SecurityTools {

    /**
     * checks users permission to read
     * @param char(3) $access access code
     * @param number $author author users id (in table users in database)
     * @param number $group groups id (in table usergroups in database)
     * @return boolean
     */
    public static function check_read ( $access, $author, $group )
    {
    	if ( empty ( $access ) || empty ( $author ) || empty ( $group ) ) {
    	    return false;
    	}
    	if ( $author == User::session () -> getId () || // user is author, so he can sure read
    	     User::session () -> getPrimaryGroup () == 1 ||  // administrator can read everything
    	     ( User::session () -> getPrimaryGroup () == 2 && $group != 1 ) || // webmaster can read anything if it doesn`t belong to administrators
    	     substr ( $access, -1 ) == "r" || // in access is that all users can read it
    	     substr ( $access, -1 ) == "w" || // in access is that all users can write, so they can read it too
    	     ( substr ( $access, -2, 1 ) == "r" || substr ( $access, -2, 1 ) == "w" ) && ( Database::connection () -> getResult ( "SELECT groupid FROM membersofgroups WHERE userid=". User::session () -> getId () ." AND groupid=". (int) $group .";" ) ) ) { // user is member of needed usergroup and that user group can read
    	    return true;
    	} else {
    	    return false;
    	}
    } // end check_read

    /**
     * checks users permission to write
     * @param char(3) $access access code
     * @param number $author author users id (in table users in database)
     * @param number $group groups id (in table usergroups in database)
     * @return boolean
     */
    public static function check_write ( $access, $author, $group )
    {
    	if ( User::session () -> getPrimaryGroup () == 1 || // administrator can write everywhere
    	     ( User::session () -> getPrimaryGroup () == 2 && $group != 1 ) || // webmaster can write anywhere if that doesn`t belong to administrators
    	     ( $author == User::session () -> getId () && substr ( $access, -3, 1 ) == "w" ) ||  // user is author and he can write
    	     substr ( $access, -1 ) == "w" || // in access is that all users can write, so they can read it too
    	     ( substr ( $access, -2, 1 ) == "w" && Database::connection () -> getResult ( "SELECT groupid FROM membersofgroups WHERE userid=". User::session () -> getId () ." AND groupid=". (int) $group .";" ) ) ) {
    	    return true;
    	} else {
    	    return false;
    	}
    } // end check_write
 
}
?>