<?php
/**
 * Class Settings gives to developer an interface to work simply with settings
 * throught SQLite files.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class Settings {

    private $dbh;
    private $cache = array ();

    /**
     * gets access to SQLite database in target file. If the file doesn't exists it will create it including database settings table
     * @param String $sqlite_database database SQLite file name
     */
    public function __construct ( $sqlite_database )
    {
    	if ( file_exists ( "sqlite/$sqlite_database.db" ) ) {
	    if ( !$this -> dbh = @sqlite_open ( "sqlite/$sqlite_database.db", 0666, $sqlite_error )  ) {
	    	Error::ticket () -> notice ( 'cor00023', 'in '.__FILE__.' on line '.__LINE__, $sqlite_error );
	    }
	} else { // if sqlite file isn`t created constructor create it
	    if ( !$this -> dbh = @sqlite_open ( "sqlite/$sqlite_database.db", 0666, $sqlite_error )  ) {
		Error::ticket () -> notice ( 'cor00024', 'in '.__FILE__.' on line '.__LINE__, $sqlite_error );	    }
	    if ( !$this -> createSettingsTable () ) {
	    	Error::ticket () -> notice ( 'cor00025', 'in '.__FILE__.' on line '.__LINE__, $sqlite_error );
	    }
	}
    }

    private function createSettingsTable ()
    {
    	$query = "
	CREATE TABLE settings
	(
    	    name PRIMARY KEY,
    	    value
	);";
	
	if ( !$this -> query ( $query ) ) {
	    return false;
	}
	
	return true;
    }

    private function query ( $query )
    {
	if ( $this -> result = @sqlite_query ( $query, $this -> dbh ) ) {
    	    return $this -> result;
    	} else {
    	    Error::ticket () -> notice ( 'cor00026', 'in '.__FILE__.' on line '.__LINE__, $query ." : ". sqlite_error_string ( @sqlite_last_error ( $this -> dbh ) ) );
	    return null;
    	}
    }

    /**
     * gets a setting
     * @param String $name name of the setting
     * @return mixed string as result or false
     */
    public function getSetting ( $name )
    {
    	$name = sqlite_escape_string ( $name );
        if ( isset ( $this -> cache [ $name ] ) ) {
            return $this -> cache [ $name ];
        }
    	if ( $result = @sqlite_single_query ( $this -> dbh, "SELECT value FROM settings WHERE name='$name';" ) ) {
            $this -> cache [ $name ] = $result;
    	    return $result;
    	} else {
            return false;
    	}
    }

    /**
     * Gets an array containing all settings in the SQLite file
     * @return mixed associative array as result or false
     */
    public function getAllSettings ()
    {
    	$query = $this -> query ( "SELECT name, value FROM settings;" );
        while ( $entry = @sqlite_fetch_array ( $query, SQLITE_ASSOC ) ) {
    	    $array_of_results [ $entry [ "name" ] ] = $entry [ "value" ] ;
        }
        if ( is_array ( $array_of_results ) ) {
            return $array_of_results;
        } else {
            return false;
        }
    }

    /**
     * updates or creates a setting
     * @param String $name name of the setting
     * @param String $value settings value
     * @return boolean
     */
    public function updateSetting ( $name, $value )
    {
        $name = sqlite_escape_string ( $name );
    	$value = sqlite_escape_string ( $value );
    	if ( $this -> query ( "REPLACE INTO settings ( name, value ) VALUES ( '$name', '$value' );" ) ) {
            $this -> cache [ $name ] = $value;
    	    return true;
    	} else {
    	    Error::ticket () -> notice ( 
    	    sqlite_error_string ( 'cor00026', 'in '.__FILE__.' on line '.__LINE__, @sqlite_last_error ( $this -> dbh ) ) );
    	    return false;
    	}
    }

    /**
     * checks connection
     * @return boolean
     */
    public function is_connected ()
    {
    	return is_resource ( $this -> dbh );
    }

    /**
     * runs all SQLite queries from a file
     * @param String $path_to_file path to target file
     * @return mixed false or associative array:
     * "errors" => number of error queries,
     * "ok" => number of ok queries,
     * "error sqls" => array with all error sqls
     */
    function SQLiteFromFile ( $path_to_file )
    {
    	$result = array ( "errors" => 0, "ok" => 0, "error sqls" => "" );

	@set_time_limit ( 0 );

	if ( !( $handle = @fopen ( $path_to_file, 'r' ) ) ) {
	    return false;
	}

	$sql = '';
	while ( !feof ( $handle ) ) {
	    $line = fgets ( $handle );
	    if ( substr ( ltrim ( $line ), 0, 2 ) !== '--' ) {
	    	$sql .= $line;
	    	if ( substr ( rtrim ( $line ), -1 ) === ';' ) {
		    if ( @sqlite_query ( $sql, $this -> dbh ) ) {
			$result [ "ok" ] = $result [ "ok" ] + 1;
		    } else {
    	    	    	$result [ "errors" ] = $result [ "errors" ] + 1;
    	    	    	$result [ "error sqls" ][] = $sql;
    	    	    }
		    $sql = '';
		}
	    }
	}
	fclose ( $handle );

    	return $result;
    } // end SQLiteFromFile

}
?>