<?php
/**
 * Class UnknownUser is called by class User for managing users without login
 * and any type of tracking.
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class UnknownUser implements UserInterface {

    /**
     * Class UnknownUser is called by class User for managing users without login and any type of tracking.
     */
    public function __construct () {
    } // end __construct

    /**
     * gets users full name
     * @return String
     */
    public function getFullName ()
    {
    	return _( "Neznámý Uživatel" );
    } // end getFullName

    /**
     * gets users id in database table users
     * @return number
     */
    public function getId ()
    {
    	return 0;
    } // end getId

    /**
     * gets users nick
     * @return String
     */
    public function getNick ()
    {
    	return _( "Anonym" );
    } // end getNick

    /**
     * gets name of the class
     * @return String
     */
    public function getType ()
    {
    	return "UnknownUser";
    } // end getType

    /**
     * gets primary group id in database table usergroups
     * @return number 1-4
     * 1: administrators
     * 2: webmasters
     * 3: editors
     * 4: users
     */
    public function getPrimaryGroup ()
    {
    	return 0;
    } // end getPrimaryGroup

    /**
     * gets users prefered language
     * @return char(2)
     */
    public function getLanguage ()
    {
    	return Localization::session () -> getLanguage ();
    } // end getLanguage

    /**
     * gets users e-mail
     * @return String
     */
    public function getEmail ()
    {
    	return 0;
    } // end getEmail

}

?>